

/*
	// !FlipTabView
	Provides a a view that provides 'flip' tabs that are located on
	the left side of the view (could be right as well?)
	
	Provides standard CSS Values that can be used to customize the view.

	CSS Class
	.flip-tab
	.flip-tab-rows-container (a div containing the tabs)

	Uses the following fClass.NSView subclasses to constuct the final view.
	- FlipTabViewTabRowView
	.ftv-tab-row
	The view representing the tab.

	- FlipTabViewViewPortView
	.ftv-view-port
	The view port area, typically this can be a view/per tab, however for some tabs, this may
	be a shared view accross multiple tabs
	
	----------------------
	Delagete Methods
	----------------------
	-flipTabBarShouldSelectTab(FlipTabViewInstance, FlipTabViewRowViewInstance)
	-flipTabBarDidSelectTab(FlipTabViewInstance, FlipTabViewRowViewInstance)
*/
fClass.WBView.extend("FlipTabView", { ivars : {

tabs:[],

//Should be set by a delegate wishing to receive various delegate calls
delegate:null,

//The view port currently displayed
m__tabViewPortView:null,

tabViewPortViewContainer:null,
tabViewTabsContainer:null,
overflowTabs:[]

}, methods : {

initWithDelegate:function(DelegateInstance) {
	if(this.initWithTag("div")) {
		this.delegate = DelegateInstance;
		
		this.addCSSClass("flip-tab");
		
		//1. Add the tab row's container
		this.tabViewTabsContainer = fClass.WBView.alloc().initWithTag('div');
		this.tabViewTabsContainer.addCSSClass('flip-tab-rows-container');
		this.addSubview(this.tabViewTabsContainer);
		
		//2. Add the view port container
		this.tabViewPortViewContainer = fClass.WBView.alloc().initWithTag('div');
		this.tabViewPortViewContainer.addCSSClass('flip-tab-view-port-container');
		this.addSubview(this.tabViewPortViewContainer);
		
		this.tabViewPortViewContainer.dom.click(function(e) {
			var classes = $(e.target).attr('class');
			if(classes == 'scroll-view' || classes == 'tools-view')
				endEditTask();
		});
		
		//We need this becouse their's a bug in WBResponder not creating this event listener.
		var that = this;
		$(window).resize(function(e) {
			that.windowLiveResize(e);
		});
		
		//Resizer Widget.
		//this.resizerWidget = fClass.WBView.alloc().initWithTag('div');
		//this.resizerWidget.addCSSClass('resize-handle');
		//this.addSubview(this.resizerWidget);
		
		//$(".flip-tab-rows-container").jqResize('.resize-handle');
	}
	return this;
},

//-------- Public ----------//

//returns the tab's index
addTab:function(FlipTabViewRowViewInstance) {
	if(FlipTabViewRowViewInstance && indexof(FlipTabViewRowViewInstance, this.tabs) < 0) {
		FlipTabViewRowViewInstance.tabView = this;
		this.tabs.push(FlipTabViewRowViewInstance);
	}
},

removeTabs:function() {
	this.tabs = [];
	this.draw();
},

removeTabAtIndex:function(IndexNumber) {
	removefrom(this.tabs[IndexNumber], this.tabs);
	this.draw();
},

removeTabWithTag:function(StringTag) {
	for(var i in this.tabs) {
		if(this.tabs[i].tag == StringTag) {
			this.tabs = removefrom(this.tabs[i], this.tabs);
			this.draw();
			return;
		}
	}
},

//returns an instance of FlipTabViewTabRowVew
tabWithTag:function(StringTag) {
	for(var i in this.tabs) {
		if(this.tabs[i].tag == StringTag) {
			return this.tabs[i];
		}
	}
	return null;
},

indexOfTab:function(TabInstance) {
	var i;
	for(i=0;i<this.tabs.length;i++) {
		if(this.tabs[i] == TabInstance) {
			return i;
		}
	}
	return -1;
},

selectTab:function(FlipTabViewRowViewInstance) {
	if(FlipTabViewRowViewInstance) {
		if(this.p__fireTabShouldSelectTabOnDelegate(FlipTabViewRowViewInstance)) {
			this.p__visuallyDeselectAllTabs();
			this.p__visuallySelectTab(FlipTabViewRowViewInstance);
			this.p__fireTabDidSelectTabOnDelegate(FlipTabViewRowViewInstance);
		}
	} else {
		this.p__visuallyDeselectAllTabs();
	}
},

selectTabWithIndex:function(IndexNumber) {
	var tab = this.tabs[IndexNumber];
	if(tab) {
		this.selectTab(tab);
	}
},

selectTabWithTag:function(StringTag) {
	var tab = this.tabWithTag(StringTag);
	if(tab) {
		this.selectTab(tab);
	}
},

visuallySelectTabWithIndex:function(IndexNumber) {
	var tab = this.tabs[IndexNumber];
	if(tab) {
		this.p__visuallyDeselectAllTabs();
		this.p__visuallySelectTab(tab);
	}
},

visuallySelectTabWithTag:function(tag) {
	var tab = this.tabWithTag(tag);
	if(tab) {
		this.p__visuallyDeselectAllTabs();
		this.p__visuallySelectTab(tab);
		return this.indexOfTab(tab);
	}
	//returns the selected tab index
	return -1;
},

setTabViewPortView:function(FlipTabViewViewPortViewInstance) {
	//first remove the old ones.
	if(this.tabViewPortView() != null) {
		NSLog(this.m__tabViewPortView);
		this.m__tabViewPortView.removeFromSuperview();
	}
	FlipTabViewViewPortViewInstance.willAddToFlipView(this);
	this.m__tabViewPortView = FlipTabViewViewPortViewInstance;
	FlipTabViewViewPortViewInstance.tabView = this;
	this.tabViewPortViewContainer.addSubview(FlipTabViewViewPortViewInstance);
	FlipTabViewViewPortViewInstance.didAddToFlipView(this);
},

tabViewPortView:function() {
	return this.m__tabViewPortView;
},

adjustTabsToFit:function() {
	if(this.tabs.length>0) {
		this.p__adjustOverflowTabVisibility();
	}
},

hideTabBar:function() {
	this.tabViewTabsContainer.hide();
},

showTabBar:function() {
	this.tabViewTabsContainer.show();
},

//-------- As WBView --------//

draw:function() {
	this.performSuper(this, "draw");
	
	//Redraw the tab views.
	this.tabViewTabsContainer.removeAllSubviews();
	var l = this.tabs.length;
	for(var i in this.tabs) {
		this.tabs[i].removeCSSClass("first");
		this.tabs[i].removeCSSClass("mid");
		this.tabs[i].removeCSSClass("last");
		
		if(i == 0) {
			this.tabs[i].addCSSClass("first");	
		} else if(i != l-1) {
			this.tabs[i].addCSSClass("mid");
		} else {
			this.tabs[i].addCSSClass("last");
		}
		
	
		this.tabs[i].dom[0].style.zIndex = l-i;
		this.tabViewTabsContainer.addSubview(this.tabs[i]);
	}
},

//---------- As WBResponder -----------//
windowLiveResize:function(e) {
	this.p__adjustOverflowTabVisibility(true);
},

//-------- Private ----------//

p__adjustOverflowTabVisibility:function(AnimateBool) {	
	//1. get current visible height.
	full = this.dom.height();
	if(!this.lastHieght)
		this.lastHieght = full;
	
	var nArr = diffFromArrayNotIn(this.tabs, this.overflowTabs);
	var itemHeight = this.tabs[0].dom.outerHeight();
	//var pt = this.tabViewTabsContainer.dom[0].style.paddingTop;
	//var pb = this.tabViewTabsContainer.dom[0].style.paddingBottom;
	
	var extra = 0;
	if(!AnimateBool) {
		extra = itemHeight;
	}
	
	AnimateBool = false;
	
	//1. calculate current height of all tabs together, of the flip-tab-rows-container
	
	var currentHeight = this.tabs[0].dom.parent().outerHeight();
		
	//2. calculate current height + the static item
	if(full < currentHeight - 5) {		//(itemHeight*(nArr.length))+pt+pb-10	
		nArr.pop(); //pop the more tab
		
		var withinHeight = false;	
		while(!withinHeight) {
			var item = nArr.pop();
			this.overflowTabs.push(item);
			if(AnimateBool) {
				//hide the badge first
				//item.hideBadge(false);
				item.dom.slideUp('fast');
			} else {
				item.hide();
			}
			
			currentHeight -= item.dom.outerHeight();
			
			if(!AnimateBool) {
				//var ih = item.dom.height()*nArr.length;
			
				//Provides performace bosts, and should not be animated on load
				if((full+5 > currentHeight)) { //(itemHeight*(nArr.length))+pt+pb
					withinHeight = true;
				}
			} else {
				withinHeight = true
			}	
		}
	} else 
	//Its being dragged larger and a new tab will fit
	if(this.overflowTabs.length > 0 && this.lastHieght < full) { //(itemHeight*(nArr.length+1))+pt+pb-10
		if(full > currentHeight + this.overflowTabs[this.overflowTabs.length-1].dom.outerHeight()) {
			var withinHeight = false;
			while(!withinHeight && this.overflowTabs.length > 0) {
				var item = this.overflowTabs.pop(item);
				if(AnimateBool) {
					//item.badge.hide();
					item.dom.slideDown('fast');
				} else {
					item.show();
				}
				
				currentHeight += item.dom.outerHeight();
				
				if(!AnimateBool) {			
					//Provides performace bosts, and should not be animated on load
					if((full < currentHeight+10)) { //(itemHeight*(nArr.length))+pt+pb
						withinHeight = true;
					}
				} else {
					withinHeight = true
				}	
			}
		}
	}
	this.lastHieght = full;
},

p__visuallyDeselectAllTabs:function() {
	for(var i in this.tabs) {
		if(this.tabs[i].selected) {
			this.tabs[i].setSelected(false);
		}
	}
},

p__visuallySelectTab:function(FlipTabViewRowViewInstance) {
	//Note, the FlipTabViewRowViewInstance could be a copy from an overflow, we need to get the real one.
	var real = this.tabWithTag(FlipTabViewRowViewInstance.tag);
	if(real) {
		real.setSelected(true);
	}
},

p__fireTabShouldSelectTabOnDelegate:function(FlipTabViewRowViewInstance) {
	if(this.delegate && this.delegate.flipTabBarShouldSelectTab) {
		return this.delegate.flipTabBarShouldSelectTab(this, FlipTabViewRowViewInstance);
	}
	return true;
},

p__fireTabDidSelectTabOnDelegate:function(FlipTabViewRowViewInstance) {
	if(this.delegate && this.delegate.flipTabBarDidSelectTab) {
		this.delegate.flipTabBarDidSelectTab(this, FlipTabViewRowViewInstance);
	}
},

}});


/*
	// !FlipTabViewTabRowView
		CSSClass: 		.ftv-tab-row
		Opt Classes:	.selected
*/
fClass.WBView.extend("FlipTabViewTabRowView", { ivars : {

//A unique tag that can be applied to the tab.
tag:"",
selected:false,
tabView:null,
representedObject:null,
overflowable:true

}, methods : {

initWithRepresentedObject:function(ObjectInstance) {
	if(this.initWithTag("div")) {
		if(ObjectInstance)
			this.representedObject = ObjectInstance;
		
		this.addCSSClass("ftv-tab-row");
		this.setDrawsOutOfBounds();
		
		//Actions
		var that = this;
		this.dom.click(function(e) {
			that.mouseClicked(e);
		});
		
		this.dom.mousedown(function(e) {
			that.mouseDown(e);	
		});
		
		this.dom.mouseup(function(e) {
			that.mouseUp(e);	
		});
		
		//Add the badge (its actuall drawn outside of self)
		this.badge = fClass.WBView.alloc().initWithTag('div');
		this.badge.addCSSClass('alert-badge');
		this.badge.setAbsolute();
		this.addSubview(this.badge);
		this.badge.hide();
	
		//No defualt color, as it may be setup in css
	}
	return this;
},

mutableCopy:function() {
	var cp = this.performSuper(this, 'mutableCopy');
	cp.badge.dom = this.badge.dom.clone();
	
	var that = cp;
	that.dom.click(function(e) {
		that.mouseClicked(e);
	});
	
	that.dom.mousedown(function(e) {
		that.mouseDown(e);	
	});
	
	that.dom.mouseup(function(e) {
		that.mouseUp(e);	
	});
	
	//Set the original
	cp.original = this;
	
	return cp;
},

//Does not fire any events
setSelected:function(BOOLFlag) {
	this.selected = BOOLFlag;
	if(BOOLFlag) {
		this.oldZ = this.dom[0].style.zIndex;
		this.dom[0].style.zIndex = 35;
		
		this.oldBackground = this.dom[0].style.background;
		this.dom[0].style.background = null;
		
		this.addCSSClass('selected');
	} else {
		this.dom[0].style.zIndex = this.oldZ;
		this.dom[0].style.background = this.oldBackground;
		this.removeCSSClass('selected');
	}
},

setBadgeCount:function(BadgeCount) {
	this.badgeCount = BadgeCount;
	
	if(!BadgeCount || BadgeCount < 1) {
		this.badge.hide();
	} else {
		this.badge.dom.html(BadgeCount);
		this.badge.show();
	}
},

showBadge:function(animate) {
	if(this.badgeCount && this.badgeCount > 0) {
		if(animate) {
			this.badge.dom.fadeIn();
		} else {
			this.badge.show()
		}
	} 
},

hideBadge:function(animate) {
	if(this.badgeCount && this.badgeCount > 0) {
		if(animate) {
			this.badge.dom.fadeOut();
		} else {
			this.badge.hide()
		}
	}
},

setBackgroundColor:function(WBColorInstance) {
	this.backgroundColor = WBColorInstance;
	this.p__setBackgroundGradientWithColorAndNudge(WBColorInstance, 0.05);	
},

//Actions
mouseClicked:function(event) {
	if(!this.selected) {
		this.tabView.selectTab(this);
	}
},

mouseDown:function(event) {
	if(!this.selected) {
		if(this.backgroundColor) {
			this.p__setBackgroundGradientWithColorAndDarken(this.backgroundColor, 0.06);
		}
	}
	
},

mouseUp:function(event) {
	if(!this.selected) {
		if(this.backgroundColor) {
			this.p__setBackgroundGradientWithColorAndNudge(this.backgroundColor, 0.05);
		}
	}
},

mouseExited:function(event) {
	if(!this.selected) {
		this.mouseUp(event);
	}
},

p__setBackgroundGradientWithColorAndNudge:function(WBColorInstance, nudge) {
	var start = WBColorInstance.mutableCopy();
	var stop = 	WBColorInstance.mutableCopy();

	//make start a bit lighter
	start.red = start.red - nudge;
	start.green = start.green - nudge;
	start.blue = start.blue - nudge;

	//make end a bit darker
	stop.red = stop.red + nudge
	stop.green = stop.green + nudge;
	stop.blue = stop.blue + nudge;

	var gradient = "-webkit-gradient(linear,"+
   						"left bottom,"+
   					 	"left top,"+
    					"color-stop(0,"+ start.cssValue() +"),"+
    					"color-stop(1.0,"+ stop.cssValue() +"))";
	
	this.dom[0].style.background = gradient;
},

p__setBackgroundGradientWithColorAndDarken:function(WBColorInstance, nudge) {
	var start = WBColorInstance.mutableCopy();
	var stop = 	WBColorInstance.mutableCopy();

	//make start a bit lighter
	start.red = start.red - nudge;
	start.green = start.green - nudge;
	start.blue = start.blue - nudge;

	//make end a bit darker
	stop.red = stop.red - nudge*2
	stop.green = stop.green - nudge*2;
	stop.blue = stop.blue - nudge*2;

	var gradient = "-webkit-gradient(linear,"+
   						"left bottom,"+
   					 	"left top,"+
    					"color-stop(0,"+ start.cssValue() +"),"+
    					"color-stop(1.0,"+ stop.cssValue() +"))";
	
	this.dom[0].style.background = gradient;
}

}});

/*
	// !FlipTabViewViewPortView
		CSSClass: 		.ftv-view-port
		
		Adds support for button 'tools' that are displayed at the bottom 
		of the view in a tool port box. see fClass.ToolItem
*/
fClass.WBView.extend("FlipTabViewViewPortView", { ivars : {
	
leftTools:[],
rightTools:[],
toolsContainer:null
	
}, methods: {

initWithTag:function(tag) {
	if(this.performSuper(this, 'initWithTag', [tag])) {
		this.toolsContainer = fClass.WBView.alloc().initWithTag('div');
		this.toolsContainer.addCSSClass('tools-view');
		this.toolsContainer.left = fClass.WBView.alloc().initWithTag('div');
		this.toolsContainer.left.addCSSClass('left-tools');
		this.toolsContainer.addSubview(this.toolsContainer.left);
		this.toolsContainer.right = fClass.WBView.alloc().initWithTag('div');
		this.toolsContainer.right.addCSSClass('right-tools');
		this.toolsContainer.addSubview(this.toolsContainer.right);
		//this.addSubview(this.toolsContainer);
		//Not added to view heighky by defualt.
	}
	return this;
},

willAddToFlipView:function(FlipTabViewInstance) {
	//for subclasses
},

didAddToFlipView:function(FlipTabViewInstance) {
	//Subclasses should call super if they want the tool bar displayed, also it should
 	//be called after any adding of subviews

	if(this.leftTools.length > 0 || this.rightTools.length > 0) {
		this.addSubview(this.toolsContainer);
	} else {
		this.removeSubview(this.toolsContainer);
	}
},

addLeftTool:function(ToolItem) {
	this.leftTools.push(ToolItem);
	//this.toolsContainer.left.removeAllSubviews();
	
	//var i, c = this.leftTools.length;
	//for(i=0; i<c; i++) {
	//	var nt = this.leftTools[i];
		this.toolsContainer.left.addSubview(ToolItem);
	//}
	return ToolItem;
},

addRightTool:function(ToolItem) {
	this.rightTools.push(ToolItem);
	//this.toolsContainer.right.removeAllSubviews();
	
	//var i, c = this.rightTools.length;
	//for(i=0; i<c; i++) {
	//	var nt = this.rightTools[i];
		this.toolsContainer.right.addSubview(ToolItem);
	//}
	return ToolItem;
},

removeTools:function() {
	this.removeLeftTools();
	this.removeRightTools();
},

removeLeftTools:function() {
	this.leftTools = [];
	this.toolsContainer.left.removeAllSubviews();
},

removeRightTools:function() {
	this.rightTools = [];
	this.toolsContainer.right.removeAllSubviews();
}

}});


/*
	// !FlipTabViewOverflowPortView
	CSSClass: .ftv-view-port > .scroll-view > .overflow

	Provides integrated overflow view for a typical 'more' button, 
	delegates are required to provide this view to the FlipView if
	a more button is clicked.
*/
fClass.FlipTabViewViewPortView.extend("FlipTabViewOverflowPortView", { ivars : {

showOverflowOnly:false,
flipView:null,
tabs:[]

}, methods : {

init:function() {
	if(this.initWithTag('div')) {
		this.scrollView = fClass.WBView.alloc().initWithTag('div');
		this.scrollView.addCSSClass('scroll-view');
		this.scrollView.setDrawsOutOfBounds();
		
		this.scrollView.content = fClass.WBView.alloc().initWithTag('div');
		this.scrollView.content.addCSSClass('overflow');
		this.scrollView.content.setDrawsOutOfBounds();
		this.scrollView.addSubview(this.scrollView.content);
		
		this.addSubview(this.scrollView);
	}
	return this;
},

didAddToFlipView:function(FlipTabViewInstance) {
	this.scrollView.content.dom.empty();
	this.flipView = FlipTabViewInstance;
	this.displayOverflowItemGrid();
},

displayOverflowItemGrid:function() {
	//Display the appropriate items
	var items = null;
	if(!this.showOverflowOnly) {
		items = this.flipView.tabs;
	} else {
		items = this.flipView.overflowTabs;
	}
	
	this.tabs = [];
	
	var l = items.length;
	for(var i in items) {
		if(i != l-1) {
			//create a real copy...
			var ntab = items[i].mutableCopy();
			
			ntab.removeCSSClass("first");
			ntab.removeCSSClass("mid");
			ntab.removeCSSClass("last");
			ntab.addCSSClass('grid');
			ntab.setFloatLeft();
		
			this.scrollView.content.addSubview(ntab);
			this.tabs.push(ntab);
		}
	}
}

}});


/*
	// !ToolItem
	Provides a tool to be displayed in the 'tools' box at the bottom of a view
*/
fClass.WBView.extend('ToolItem', { ivars: {

}, methods: {

init:function() {
	if(this.initWithTag('div')) {
		this.addCSSClass('tool-item');
		this.icon = fClass.WBView.alloc().initWithTag('img');
		this.title = fClass.WBView.alloc().initWithTag('div');
		
		this.addSubview(this.icon);
		this.addSubview(this.title);
		
		var that = this;
		$(this.dom).mousedown(function() {
			that.mouseDown();
		}).mouseup(function() {
			that.mouseUp();
		}).mouseleave(function() {
			that.mouseExit();
		});
		
		
	}
	return this;
},

initWithOptions:function(icon, title, target, action) {
	if(this.init()) {
		this.baseIcon = icon;
		
		this.setIconNamed(icon);
		this.setTitle(title);
		this.setTarget(target);
		this.setAction(action);
	}
	return this;
},

setTarget:function(target) {
	this.target = target;
},

setAction:function(action) {
	this.action = action;
	var that = this;
	this.dom.click(function() {
		that.target[action](that);
	});
},

setIconNamed:function(named) {
	this.icon.dom[0].src = named+'.png';
},

setTitle:function(title) {
	this.title.dom.html(title);
},

state:function() {
	return this.dom.hasClass('selected');
},

setState:function(flag) {
	this.flagState = flag;
	if(flag) {
		this.dom.addClass('selected');
		this.setIconNamed(this.baseIcon+"_active");
	} else {
		this.dom.removeClass('selected');
		this.setIconNamed(this.baseIcon);
	}
},

mouseDown:function() {
	this.setIconNamed(this.baseIcon+"_active");
},

mouseUp:function() {
	//if(!this.flagState) {
		this.setIconNamed(this.baseIcon);
	//}
},

mouseExit:function() {
	if(!this.flagState) {
		this.setIconNamed(this.baseIcon);
	}
}

}});

/*
	---------------------------------------------------------------------
	
*/

// !More Tab
fClass.FlipTabViewTabRowView.extend("MoreTab", { ivars: {

}, methods: {
	
init:function() {
	if(this.initWithRepresentedObject(null)) {
		this.tag = 'more';
		this.addCSSClass('more-tab');
		this.addButton = fClass.WBButtonLink.alloc().init();
		this.addButton.setId("add-project-button");
		this.addButton.setTitle("✚");
		
		/*
			Currently theirs a bug with WBControl/WBButtonLink not sending an action on click..., 
			this is a workaround - it should not be needed as the WBButtonLink defines its action
			event as click, and sets up this binding automatically...
		*/
		this.addButton.setTarget(this);
		this.addButton.setAction('addButtonClicked');
		var that = this;
		this.addButton.dom.click(function(e) {
			return that.addButtonClicked(that, e);
		});
		
		this.addSubview(this.addButton);
		
		//Updated whenever the project count changes, if count > current list then
		//it shows XX More or it shows Projects
		this.title = fClass.WBView.alloc().initWithTag("span");
		this.title.dom.html("Projects");
		this.addSubview(this.title);
		
		//The more tab is always visible
		this.overflowable = false;
		
	}
	return this;
},

addButtonClicked:function(sender, event) {
	this.tabView.delegate.showProjectCreateView();
	event.stopPropagation();
},
	
setMoreCount:function(CountOfMore) {
	if(CountOfMore < 1) {
		this.title.dom.html('See all');
	} else {
		this.title.dom.html(CountOfMore + ' More');
	}
}
	
}});

// !ProjectTab
fClass.FlipTabViewTabRowView.extend("ProjectTab", { ivars: {

}, methods: {
	
initWithProject:function(project) {
	if(this.initWithRepresentedObject(project)) {
		this.addCSSClass('project-tab');
	
		//Safari 5 Fix
		this._representedUID = project.valueForKey('uid'); 
	
		this.edit = fClass.WBView.alloc().initWithTag('img');
		this.edit.dom[0].src = 'editcog.png';
		this.edit.addCSSClass('edit-cog');
		this.addSubview(this.edit);
		this.edit.hide();
		
		var that = this;
		this.edit.dom.click(function(e) {
			that.editProjectAction();
			e.stopPropagation();
			return false;
		});
		
	
		this.img = fClass.WBView.alloc().initWithTag("div"); //was img
		this.img.addCSSClass('project-image');
		this.addSubview(this.img);
		
		//this.setImage("project_box");
	
		//Updated whenever the project count changes, if count > current list then
		//it shows XX More or it shows Projects
		this.title = fClass.WBView.alloc().initWithTag("h3");
		this.title.addCSSClass('p-title');
		this.title.dom.html('No title and lots of details'); //defualt;
		this.addSubview(this.title);
		
		//Defualt tab color
		//this.setColorValues(0.342,0.376,0.428);
		//this.setColorValues(0.621,0.676,0.730);
		//this.setColorValues(0.63,0.64,0.68);
		this.setColorValues(0.442,0.476,0.528);
	}
	return this;
},

mutableCopy:function() {	
	var cp = this.performSuper(this, "mutableCopy");
	
	//mutibleCopy doesnt really work for WBView's yet, it has some duplication bugs, this
	//is a bit of a hack-fix
	cp.dom.empty();
		
	//Copy internal doms
	cp.edit.dom = this.edit.dom.clone();
	cp.img.dom = this.img.dom.clone();
	cp.title.dom = this.title.dom.clone();
	
	//Safari 5 fix
	cp._representedUID = this._representedUID;
	
	return cp;
},

setTitle:function(TitleHTML) {
	this.titleString = TitleHTML;
	this.title.dom.html(TitleHTML);
	this.dom[0].title = TitleHTML;
},

setImage:function(ImageName) {
	this.imageName = ImageName;
	this.img.dom[0].src = "./project_images/"+ImageName+".png";
},

setColorValues:function(red, green, blue) {
	var color = fClass.WBColor.alloc().init(red, green, blue);
	this.setBackgroundColor(color);
},

setCounts:function(IncompleteCount, PendingCount) {
	
},

setChatCountBadge:function(ChatCount) {

},

editProjectAction:function() {
	this.tabView.delegate.showProjectEditView(this.representedObject);
}
	
}});

// !TasksListView
//Project List View Port (#PL1 View)
fClass.FlipTabViewViewPortView.extend("TasksListView", { ivars: {
		
//If the project titles should be displayed for mixed mode views
showsProjectTitles:false
		
}, methods: {	

init:function() {
	if(this.initWithTag('div')) {
		this.addCSSClass('view-port-with-header');
		
		/*this.header = fClass.WBView.alloc().initWithTag('div');
		this.header.addCSSClass('view-header');
		
		this.header.image = fClass.WBView.alloc().initWithTag('img');
		this.header.image.addCSSClass('view-header-icon');
		this.header.addSubview(this.header.image);
		
		this.header.title = fClass.WBView.alloc().initWithTag('h2');
		this.header.title.addCSSClass('view-header-title');
		this.header.addSubview(this.header.title);
	
		//...Header edit button controls
		
		this.addSubview(this.header);*/
		
		//Add the scroll-view.
		this.scrollView = fClass.WBView.alloc().initWithTag('div');
		this.scrollView.addCSSClass('scroll-view');
		this.scrollView.setDrawsOutOfBounds();
		
		this.scrollView.content = fClass.WBView.alloc().initWithTag('div');
		this.scrollView.content.addCSSClass('tasks');
		this.scrollView.content.addCSSClass('content');
		this.scrollView.content.setId('list-view');
		this.scrollView.content.setDrawsOutOfBounds();
		this.scrollView.addSubview(this.scrollView.content);
		
		this.addSubview(this.scrollView);
		
		//Null Defualts
		this.setHeaderImage('box');
		this.setHeaderTitle('No project info');
		this.setHeaderColor(fClass.WBColor.alloc().init(0.3,0.3,0.3));
	
		//this.addTools();
	}
	return this;
},

addTools:function() {
	this.removeTools();
	
	this.addLeftTool(fClass.ToolItem.alloc().initWithOptions('addtasktool', 'New Task', this, 'createTaskAction'));
	//this.addLeftTool(fClass.ToolItem.alloc().initWithOptions('addnotetool', 'New Note', this, 'createNoteAction'));
	
	//Not the best way to do this, but works for a quick hack until we implement a better way of closing the button.
	window.taskEditButton = this.addLeftTool(fClass.ToolItem.alloc().initWithOptions('editcogtool', 'Edit', this, 'editTaskAction'));
	this.addRightTool(fClass.ToolItem.alloc().initWithOptions('Trash_32x32', 'Delete', this, 'deleteTaskAction'));
},

didAddToFlipView:function(FlipTabViewInstance) {
	this.performSuper(this, 'didAddToFlipView', [FlipTabViewInstance]);
	/*
		This is a hack to get tool actions to work, should only need to add them in the init.
	*/
	this.addTools();
},

//----- Public -------

setHeaderTitle:function(TitleString) {
	//this.header.title.dom.html(TitleString);
},

setHeaderColor:function(WBColorInstance) {
/*	var nudge = 0.05;
	
	var start = WBColorInstance.mutableCopy();
	var stop = 	WBColorInstance.mutableCopy();
	
	//make start a bit lighter
	start.red = start.red - nudge;
	start.green = start.green - nudge;
	start.blue = start.blue - nudge;

	//make end a bit darker
	stop.red = stop.red + nudge
	stop.green = stop.green + nudge;
	stop.blue = stop.blue + nudge;

	var gradient = "-webkit-gradient(linear,"+
   						"left bottom,"+
   					 	"left top,"+
    					"color-stop(0,"+ start.cssValue() +"),"+
    					"color-stop(1.0,"+ stop.cssValue() +"))";

	this.header.dom[0].style.background = gradient;*/
},

setHeaderImage:function(ImageName) {
	//this.header.image.dom[0].src = "./project_images/"+ImageName+".png";
},

createTaskAction:function(sender) {
	TaskListController.newTaskAction_("");
},

createNoteAction:function(sender) {
	TaskListController.newNoteAction_("");
},

editTaskAction:function(sender) {
	if(!sender.state()) {
		editTask(GetSelectedTask().valueForKey('uid'));
		sender.setState(true);
	} else {
		endEditTask();
		sender.setState(false);
	}
},

deleteTaskAction:function(sender) {
	NSLog('should delete');
	TaskListController.deleteAction_("");
},

/*
	Clean out any row attributes classes
*/
p_cleanTaskRow:function(RowElement) {
	RowElement.removeClass('overdue');
	RowElement.removeClass('anytime');
	RowElement.removeClass('today');
	RowElement.removeClass('complete');
	RowElement.removeClass('filed');
	
	RowElement.removeClass('pending');
	RowElement.removeClass('unread');
},

p_setTaskRowTitle:function(row, task, uid) {	
	if(task.title) {
		row.find('.title').html(parsedTaskTitle(task.title)).attr('bnd', 'Task/'+uid);
	} else {
		//The row is considered 'pending' for the purpose of creating a new one, no bindings yet...
		row.attr('state', 'pending');
	}	
},

p_setTaskRowSecondaryTitle:function(row, task) {
	//Set the project title (Only shown under the Search tab)
	var pTitleEl = row.find('.project-title');
	var titleEl = row.find('.title');
	
	//Set share state classes to row.
	var shareState = task.shareState;
	row.attr('shareState', shareState);
	if(shareState < 10) {
		row.addClass('unread');
	} else if(shareState < 20) {
		row.addClass('pending');
	}
	
	//If incomming, show sender, else show project title only if this.showsProjectTitles.
	if(task.isIncomming) {
		pTitleEl.html("<img class='from-img' src='shared_cell_image_to_me.png'/>"+task.ownerName);
	} else {
		if(this.showsProjectTitles) {
			pTitleEl.text(task.projectTitle);
		} else {
			pTitleEl.text('');
		}
		
		//Check if its mine to show the share indicator
		if(!task.mine) {
			titleEl.append("<span class='not-mine'><img src='shared_cell_image_to_me.png' title='"
				+task.ownerName
				+" is sharing this task with you." 
				+"'/></span>");
		} else if(task.shared) {
			titleEl.append("<span class='not-mine'><img src='shared_cell_image_to_me.png' title='You are the owner of this task.'/></span>");
		} else {
			titleEl.find("span.not-mine").remove();
		}
	
	}
},

p_setTaskRowNoteIndicators:function(row, task) {
	//Check if theirs notes to show the notes indicator
	var titleEl = row.find('.title');
	if(task.hasNotes) {
		titleEl.append("<span class='note'><img src='note_icon.png' title='Indicates that this task has notes.' onclick='TaskListController.openTaskInExternalWindow_($(this).guid())'/></span>");
		
		//Check if theirs subtasks to show in the subtask indicator
		if(task.countOfSubtasks > 0) {
			titleEl.append("<span class='subtask-count'>✔" + task.countOfSubtasks +"</span>");
		} else {
			titleEl.find('span.subtask-count').remove();
		}
		
		//TODO: check if theirs maps, chats or other widget's that have indicator badges.
	} else {
		titleEl.find("span.note").remove();
	}	
},

//Silently fails if not editing this task.
p_setTaskRowUpdateNotes:function(taskId, task, editRow) {
	//update editor notes if needed, as only p_updateTaskRow is called, not the backing p_updateTaskEditor...
	if(DWiki.Editor && !DWiki.Editor.editing && DWiki.Editor.ref == taskId) {
		DWiki.Editor.setWikiText(task.notes);
	}
},

p_setTaskRowUpdateDueDate:function(row, task, editRow) {
	var dueDate = task.dueDate;
	
	//Potentially update the editor as well.
	//var editorDateEl = editRow.find('#optdate');
	
	if(dueDate) {
		var jsdate = Date.parse(dueDate);
		var dateEl = row.find('.date');
		var isAllDayToday;
		
		if(jsdate.isToday()) {
			//Check if its has a time for today, or its due anyime today
			if(jsdate.getHours() > 0 && jsdate.getMinutes() > 0) {
				dateEl.text(jsdate.toString('h:mm tt'));
			} else {
				dateEl.text('Today');
				isAllDayToday = true;
			}
			
			row.addClass('today');
		} else {
			dateEl.text(jsdate.toString('MMM d'));
		}
		dateEl.attr('date', jsdate.getTime());
		
		if(jsdate.isBefore(new Date()) && !isAllDayToday) {
			row.addClass('overdue');
		}
	} else {		//Anytime
		row.addClass('anytime').find('.date').text('Anytime');
	}
	
	
},

p_setTaskRowUpdateStates:function(row, task) {
	if(task.completed == true) {
		row.find('input.checkmark').attr('checked', 'checked');
		row.addClass('complete');
	} else {
		row.find('input.checkmark').removeAttr('checked');
		row.removeClass('complete');
	}
	
	if(task.status && task.status == 'Filed') {
		row.addClass('filed');
	}
},

/*
	The task object passed here is a simple javascipt proxy object of a real core data object.
*/
p_updateTaskRow:function(task, row) {
	this.p_cleanTaskRow(row);

	taskId =  task.uid;
	
	//Set the row's id to the task uid
	row.attr('id', taskId);
	
	//Setup the row title and subtitles
	this.p_setTaskRowTitle(row, task, taskId);
	this.p_setTaskRowSecondaryTitle(row, task);
	this.p_setTaskRowNoteIndicators(row, task);
		
	//Get the backing edit row, (these dont use bindings...)
	var editRow = $('.task-edit[id="'+taskId+'"]');
		
	this.p_setTaskRowUpdateNotes(taskId, task, editRow);
	this.p_setTaskRowUpdateDueDate(row, task, editRow);
	this.p_setTaskRowUpdateStates(row, task);
}

}});

/*
	// !ProjectsGridView
	Shown on more click
*/
fClass.FlipTabViewOverflowPortView.extend('ProjectsGridView', { ivars : {

filedProjects:null

}, methods : {

init:function() {
	if(this.performSuper(this, 'init')) {
		this.setId('projects-grid');
	}
	return this;
},

didAddToFlipView:function(FlipView) {
	//this.performSuper(this, 'didAddToFlipView', [FlipView]);
	this.flipView = FlipView;
	this.scrollView.content.dom.empty();
	
	var header = fClass.WBView.alloc().initWithTag('h2');
	header.addCSSClass('project-count-header');
	header.dom.html(FlipView.tabs.length-1 + ' Active Projects');
	this.scrollView.content.addSubview(header);
	
	//Does not work with .mutableCopy
	//this.displayOverflowItemGrid();
	
	var l = FlipView.tabs.length;
	for(var i in FlipView.tabs) {
		if(i != l-1) {
			var proj = FlipView.tabs[i].representedObject;
			
			//A strange error with safari 5's webkit that couses some nastyness with managed object accessing.
			if(!proj.valueForKey) {
				window.console.log('project does not exist? correcting problem...');
				proj = TaskListController.projectWithUID_(FlipView.tabs[i]._representedUID);
			}
			
			//NSLog('overflow next:'+proj.valueForKey('title'));
			var tab = fClass.ProjectTab.alloc().initWithProject(proj);
			tab.tag = proj.valueForKey('uid'); //.valueForKey('uid');
			tab.setTitle(proj.valueForKey('title'));
			tab.setBadgeCount(proj.valueForKey('dueCount'));
			tab.addCSSClass('grid');
			tab.setFloatLeft();
						
			tab.tabView = FlipView;
			this.scrollView.content.addSubview(tab);
			this.tabs.push(tab);
		}
	}
	
	//Add overflow items
	
	if(this.filedProjects.length > 0) {
	
		header = fClass.WBView.alloc().initWithTag('h2');
		header.addCSSClass('project-count-header');
		header.dom.html(this.filedProjects.length + ' Filed Projects');
		this.scrollView.content.addSubview(header);
	
		for(var i in this.filedProjects) {
			var fp = this.filedProjects[i];
			var tab = fClass.ProjectTab.alloc().initWithProject(fp);
			tab.tag = fp.valueForKey('uid');
			tab.setTitle(fp.valueForKey('title'));
			tab.setBadgeCount(fp.valueForKey('dueCount'));
			tab.addCSSClass('grid');
			tab.setFloatLeft();
			
			//This typically gets set when its added to the flipView, however becouse
			//we are manageing these we need to add this here
			tab.tabView = FlipView;
			
			this.scrollView.content.addSubview(tab);
			this.tabs.push(tab);
		}
	}
		
	//Add the edit controls
	var i, c = this.tabs.length;
	for(i = 0; i < c; i++) {
		var t = this.tabs[i];
		t.edit.show();
	}
}

}});

/*
	// !ProjectsEditView
	Shared as both an edit view and a create view for creating projects.
*/
fClass.FlipTabViewViewPortView.extend('ProjectsEditView', { ivars : {

}, methods : {

init:function() {
	if(this.initWithTag('div')) {
		this.setId('project-edit-view');
		this.setDrawsOutOfBounds();
	
		this.header = fClass.WBView.alloc().initWithTag('h1');
		//this.header.img = fClass.WBView.alloc().initWithTag('img');
		//this.header.img.setFloatLeft();
		//this.header.addSubview(this.header.img);
		//this.header.img.dom[0].src = 'note_icon.png'; //TODO: change me.
		this.header.dom.html('<span>Edit Project</span>');
		this.addSubview(this.header);
		
		this.title = fClass.WBView.alloc().initWithTag('div');
		this.title.addCSSClass('option');
		this.title.label = fClass.WBView.alloc().initWithTag('div');
		//this.title.label.setFloatLeft();
		this.title.label.addCSSClass('label');
		this.title.label.dom.html('Title');
		this.title.addSubview(this.title.label);
		this.title.field = fClass.WBView.alloc().initWithTag('div');
		this.title.field.addCSSClass('textfield');
		this.title.field.addCSSClass('ph');
		this.title.field.setDrawsOutOfBounds();
		this.title.field.dom.attr('placeholder', 'Project title...');
		this.title.addSubview(this.title.field);
		
		this.title.field.dom.blur();
		this.title.setDrawsOutOfBounds();
		this.addSubview(this.title);
		
		/*this.color = fClass.WBView.alloc().initWithTag('div');
		this.color.addCSSClass('option');
		this.color.label = fClass.WBView.alloc().initWithTag('div');
		this.color.label.addCSSClass('label');
		this.color.label.dom.html('Color');
		this.color.addSubview(this.color.label);
		this.color.well = fClass.WBView.alloc().initWithTag('div');
		this.color.well.addCSSClass('colorwell');
		this.color.addSubview(this.color.well);
		this.addSubview(this.color);
		
		this.image = fClass.WBView.alloc().initWithTag('div');
		this.image.addCSSClass('option');
		this.image.label = fClass.WBView.alloc().initWithTag('div');
		this.image.label.dom.html('Picture');
		this.image.label.addCSSClass('label');
		this.image.addSubview(this.image.label);
		this.image.well = fClass.WBView.alloc().initWithTag('div');
		this.image.well.addCSSClass('imagewell');
		this.image.addSubview(this.image.well);
		this.image.well.img = fClass.WBView.alloc().initWithTag('img');
		this.image.well.img.dom[0].src = './project_images/box.png';
		this.image.well.addSubview(this.image.well.img);
		this.addSubview(this.image);*/
		
		this.addSubview(fClass.WBView.alloc().initWithTag('hr'));
		
		this.ical = fClass.WBView.alloc().initWithTag('div');
		this.ical.addCSSClass('option');
		this.ical.addCSSClass('checkfield');
		this.ical.label = fClass.WBView.alloc().initWithTag('div');
		this.ical.label.dom.html('iCal');
		this.ical.label.addCSSClass('label');
		this.ical.addSubview(this.ical.label);
		this.ical.check = fClass.WBView.alloc().initWithTag('input');
		this.ical.check.dom.attr('type', 'checkbox');
		this.ical.check.addCSSClass('clabel');
		this.ical.addSubview(this.ical.check);
		this.addSubview(this.ical);
		
		this.filed = fClass.WBView.alloc().initWithTag('div');
		this.filed.addCSSClass('option');
		this.filed.addCSSClass('checkfield');
		this.filed.label = fClass.WBView.alloc().initWithTag('div');
		this.filed.label.dom.html('Filed');
		this.filed.label.addCSSClass('label');
		this.filed.addSubview(this.filed.label);
		this.filed.check = fClass.WBView.alloc().initWithTag('input');
		this.filed.check.dom.attr('type', 'checkbox');
		this.filed.check.addCSSClass('clabel');
		this.filed.addSubview(this.filed.check);
		this.addSubview(this.filed);
		
	}
	return this;	
},

p__setupCreateEvents:function() {
	var that = this;
	this.title.field.dom.keydown(function(e) {
		if(e.keyCode == '13') {
			e.preventDefault();
			that.saveToolAction();
		}
	});
},

p__setupEditEvents:function() {
	var that = this;
	this.title.field.dom.blur(function() {
		that.projectTitleChanged();
	}).keydown(function(e) {
		if(e.keyCode == '13') {
			e.preventDefault();
			that.saveToolAction();
		}
	});
	
	this.ical.check.dom.click(function() {
		that.projectiCalSyncChanged();
	});
	
	this.filed.check.dom.click(function() {
		that.projectFiledChanged();
	});
},

p__setupEditTools:function() {
	this.p__setupCreateTools();
	this.addLeftTool(fClass.ToolItem.alloc().initWithOptions('Trash_32x32', 'Delete', this, 'deleteToolAction'));
},

p__setupCreateTools:function() {
	this.removeTools();
	this.addRightTool(fClass.ToolItem.alloc().initWithOptions('./project_images/drive', 'Save', this, 'saveToolAction'));
},

setupForTests:function() {
	this.p__setupEditTools();
},

setProject:function(Project) {
	this.project = Project;
	if(this.project) {
		this.header.dom.find('span').text('Edit Project');
		this.filed.show();		
		
		this.title.field.dom.html(Project.valueForKey('title'));
	
		//iCal Synced?
		if(Project.valueForKey('calSynced')) {
			this.ical.check.dom.attr('checked', 'checked');
		} else {
			this.ical.check.dom.attr('checked', "");
		}
	
		//Filed?
		var state = Project.valueForKey('state');
		if(state && state == 'filed') {
			this.filed.check.dom.attr('checked', 'checked');
		} else {
			this.filed.check.dom.attr('checked', "");
		}
		
		this.p__setupEditTools();
		this.p__setupEditEvents();
	} else {
		this.header.dom.find('span').text('Create Project');
		this.filed.hide();
		this.title.field.dom.html("");
		
		this.p__setupCreateTools();
		this.p__setupCreateEvents();
	}
},

didAddToFlipView:function(FlipTabView) {
	this.performSuper(this, 'didAddToFlipView', [FlipTabView]);
},

projectTitleChanged:function() {
	var title = this.title.field.dom.text();
	
	//Handles both edit and create project modes
	if(this.project) {
		//The ProjectListUIController
		ProjectListController.projectEndTitleEditing_title_(this.project.valueForKey('uid'), title);
	}
},

projectiCalSyncChanged:function() {
	var state = this.ical.check.dom.attr('checked');
	if(this.project) {
		if(state) {		
			if(this.project.valueForKey('calSynced') == false) {
				this.project.setValueForKey(true, 'calSynced');
				//Currently in ApplicationTabProject.js legacy js
				ProjectListController.resynciCalActionForProjectWithIdentifier_(this.project.valueForKey('uid'));
			}
			
		} else {
			this.project.setValueForKey(false, 'calSynced');
		}
	}
},

projectFiledChanged:function() {
	var state = this.filed.check.dom.attr('checked');
	if(this.project) {
		//NSLog("new state for project:"+state+ " title:"+GetProject(guid).valueForKey('title'));
		if(state == 'true' || state == true) {
			this.project.setValueForKey('filed', 'state');
		} else {
			this.project.setValueForKey('normal', 'state');
		}
		ProjectListController.projectFiledStateChanged_(this.project.valueForKey('uid'));
	}
},

projectColorChanged:function() {

},

projectPictureChanged:function() {

},

saveToolAction:function() {
	
	//Force the edit
	this.title.field.dom.blur();
	
	if(!this.project) {
		var title = this.title.field.dom.text();
		var ical = this.ical.check.dom.attr('checked');
		ProjectListController.projectCreateWithTitle_color_icon_ical_(title, null, null, ical);
	} else {
		this.tabView.delegate.projectEditedAction();
	}
	
},

deleteToolAction:function() {
	if(this.project) {
		TaskListController.deleteProject_(this.project.valueForKey('uid'));
	}
}

}});

/*
	// !ProjectsTabController
	Provides the delegate control for the FlipTab - projects tab view
	
	The ProjectsTabController should be supplied an array of projects, sorted in
	order of the following logic.
	
	Projects are typically supplied with 'state' = filed, they will show up under the
	'more' tab but not in the list, even if they match the current sort prioritization.
	
	it is also supplied an array of tasks filtered according to the current subfilter for the projects
	(All), (Today), (Overdue), (Next - Tasks upcomming), (Tasks), (Notes)
	
	-Project.tasks sorting of projects.
	
	Last Unread Chat Date count,
	Overdue count,
	Priority count,
	Task count,
	Last modified count,
	Last created count.
*/
fClass.NSObject.extend("ProjectsTabController", { ivars: {

view:null,
moreView:null,
projects:null,
filedProjects:[],
tasks:null,
showsProjectTilesInRows:0,
selectedProjectIndex:-1

}, methods: {

init:function() {
	if(this.performSuper(this, 'init')) {
		//Approved project colors,
		//0.275,0.353,0.586 (gray-blue)
		//0.552,0.668,0.858 (light-blue)
		//0.275,0.353,0.586 (blue)
		//0.73,0.1,0		(red)
		//0,0.5,0 			(green)
		//0.5,0,0 			(red)
		//0.3,0.3,0.3		(dark-gray)		//Defualt?
		//0.756,0.719,0.0	(yellow)
		//0.839,0.426,0.0	(orange)
	
		/*	
			On init it shows the task list view, unless theirs no projects
			otherwise it will show the project view 'more' where the user can
			create a project
		*/
		//--Setup--
		
		this.view = fClass.FlipTabView.alloc().initWithDelegate(this);
		
		//Various View ports offered by this tabview controller.
		this.taskListViewPort = fClass.TasksListView.alloc().init();
		this.view.setTabViewPortView(this.taskListViewPort); 
		
		//Only show these in debug design view
		if(!window.TaskListController) {
			var one = fClass.ProjectTab.alloc().initWithProject();
			one.setTitle('Dejumble 2.0 Mac');
			one.tag = 'test-edit';
			this.view.addTab(one);
			one.setImage('imac');
			one.setColorValues(0.275,0.353,0.586);
			
			var two = fClass.ProjectTab.alloc().initWithProject();
			two.setTitle('Home');
			this.view.addTab(two);
			two.setImage('box');
			two.setColorValues(0.082,0.295,0.654);
			two.setBadgeCount(5);
			
			var three = fClass.ProjectTab.alloc().initWithProject();
			three.setTitle('ThinkingCode');
			this.view.addTab(three);
			three.setImage('postit');
			three.setColorValues(0.5,0,0);
			
			var red = fClass.ProjectTab.alloc().initWithProject();
			red.setTitle('FrothKit');
			this.view.addTab(red);
			red.setImage('notepad');
			red.setColorValues(0.73,0.1,0);
			
			var four = fClass.ProjectTab.alloc().initWithProject();
			four.setTitle('Dejumble 2.0 iPhone');
			this.view.addTab(four);
			four.setImage('ipod');
			four.setColorValues(0.3,0.3,0.3);
			four.setBadgeCount(2);
			
			var five = fClass.ProjectTab.alloc().initWithProject();
			five.setTitle('Hobbies');
			this.view.addTab(five);
			five.setImage('paint');
			five.setColorValues(0,0.5,0);
			
			var six = fClass.ProjectTab.alloc().initWithProject();
			six.setTitle('Color Palletes');
			this.view.addTab(six);
			six.setImage('flagred');
			six.setColorValues(0.756,0.719,0.0);
			
			var seven = fClass.ProjectTab.alloc().initWithProject();
			seven.setTitle('Orange Testing Project');
			this.view.addTab(seven);
			seven.setImage('globe');
			seven.setColorValues(0.839,0.426,0.0);
			
			var lightblue = fClass.ProjectTab.alloc().initWithProject();
			lightblue.setTitle('Light Blue');
			this.view.addTab(lightblue);
			lightblue.setImage('flaggreen');
			lightblue.setColorValues(0.552,0.668,0.858);
			lightblue.setBadgeCount(344);
			
			var brown = fClass.ProjectTab.alloc().initWithProject();
			brown.setTitle('Brown');
			this.view.addTab(brown);
			brown.setImage('heart');
			brown.setColorValues(0.50,0.21,0.0);
			
			this.view.addTab(fClass.MoreTab.alloc().init());
		}	
	}
	return this;
},

/*
	Updates the projects tab to given array of projects.
*/
setProjects:function(projectsArray) {
	this.projects = CDArray(projectsArray);
	
	this.filedProjects = [];
	
	this.view.removeTabs();
	
	var i, c = this.projects.length;
	for(i=0; i<c; i++) {
		var project = this.projects[i];
		//Dont show filed projects in main tab list, however there shown in the overflow
		if(project.valueForKey('state') != 'filed') {
			var pid = project.valueForKey('uid');
			var tab = fClass.ProjectTab.alloc().initWithProject(project);
			tab.tag = pid;
			tab.setTitle(project.valueForKey('title'));
			//NSLog('display next:'+project.valueForKey('title'));
			tab.setImage("box");
			tab.setBadgeCount(project.valueForKey('dueCount'));
			this.view.addTab(tab);
		} else {
			this.filedProjects.push(project);
		}
	}
	
	this.view.addTab(fClass.MoreTab.alloc().init());
	this.view.draw();
	this.view.adjustTabsToFit();
	
},

/*
	Updates the task's tab to given array of tasks, this is a one-to-one list of all
	tasks of currently selected project.
*/
setTasks:function(tasksArray) {
	this.tasks = tasksArray;		
	//tasksArray = CDArray(tasksArray); (for testing)
		
	var curScroll = this.listScrollPosition();
	var listView = this.taskListViewPort.scrollView.content.dom;
	listView.empty();
	
	//Should titles be shown?
	this.taskListViewPort.showsProjectTitles = this.showsProjectTilesInRows;
	
	if(this._incommingGroupDiv) {
		this._incommingGroupDiv.find('.rows').empty();
	}	

	var i, c = tasksArray.length;
	var incommingGroupDivIsVisible = false;
	for(i=0; i<c; i=i+1) {
		var task = tasksArray[i];
						
		//Now do row handling, todo, move this to fClass for modernization.
		var row = RowDivFromTemplate();
		this.taskListViewPort.p_updateTaskRow(task, row);
		//updateTaskRow_(tasksArray[i], row, this.showsProjectTilesInRows);
			
		if(this.currentContext == 'Inbox' && task.isIncomming) {
			if(!this._incommingGroupDiv) {
				this._incommingGroupDiv = GroupDivFromTemplate();
				this._incommingGroupDiv.find('span.title').text('Received Tasks');
				this._incommingGroupDiv.attr('id', 'incomming-group');
			}
			
			if(!incommingGroupDivIsVisible) {
				this._incommingGroupDiv.appendTo(listView).show();
				incommingGroupDivIsVisible = true;
			}
			
			row.appendTo(this._incommingGroupDiv.find('div.rows')).show();
		} else {
			
			row.appendTo(listView).show();
		}
	}	
	
	this.listScrollPosition(curScroll);
	TaskListController.didUpdateTaskListView();
},

//Refreshes a give row for a task. This silently fails if no task with given row is visible.
refreshTaskView:function(proxy) {
	var row = $('.task-row#'+proxy.uid);
	this.taskListViewPort.p_updateTaskRow(proxy, row);
},

selectFirstTab:function() {
	/*
		If we dont do this this 'more' tab is selected which causes the
		Inbox to not draw properly
	*/
	if(this.projects.length > 0) {
		this.view.selectTabWithIndex(0);
	}
},

layoutView:function() {
	this.view.adjustTabsToFit();
},

listScrollPosition:function(value) {
	return this.taskListViewPort.scrollView.dom.scrollTop(value);
},

//--- Flip Tab Delegate Methods ---

flipTabBarShouldSelectTab:function(FlipTabViewInstance, FlipTabViewRowViewInstance) {
	return true;
},

flipTabBarDidSelectTab:function(FlipTabViewInstance, FlipTabViewRowViewInstance) {

	
	if(FlipTabViewRowViewInstance.tag == 'test-edit') {
		this.view.setTabViewPortView(fClass.ProjectsEditView.alloc().init());
	} else if(FlipTabViewRowViewInstance.tag != 'more') {
		if(!this.view.tabViewPortView() != this.taskListViewPort) {
			this.view.setTabViewPortView(this.taskListViewPort);
		}
		this.selectedProjectIndex = FlipTabViewInstance.indexOfTab(FlipTabViewRowViewInstance);
		TaskListController.selectProject_(FlipTabViewRowViewInstance.tag);
	} else {
		this.showProjectGridView();
	}
	
	//In stead of parsing the project, we simply copy the project values from the slected tab
	//this.taskListViewPort.setHeaderTitle(FlipTabViewRowViewInstance.titleString);
	//this.taskListViewPort.setHeaderImage(FlipTabViewRowViewInstance.imageName);
	//this.taskListViewPort.setHeaderColor(FlipTabViewRowViewInstance.backgroundColor);
},

showProjectGridView:function() {
	if(!this.moreView) {
		this.moreView = fClass.ProjectsGridView.alloc().init();
	}
	if(!this.view.tabViewPortView() != this.moreView) {
		this.moreView.filedProjects = this.filedProjects;
		this.view.setTabViewPortView(this.moreView);
	}
},

showProjectEditView:function(project) {
	if(!this.editProjectView) {
		this.editProjectView = fClass.ProjectsEditView.alloc().init();
	}
	this.editProjectView.setProject(project);
	this.view.selectTab(null);
	this.view.setTabViewPortView(this.editProjectView);
},

showProjectCreateView:function() {
	if(!this.editProjectView) {
		this.editProjectView = fClass.ProjectsEditView.alloc().init();
	}
	this.editProjectView.setProject(null);
	this.view.selectTab(null);
	this.view.setTabViewPortView(this.editProjectView);
},

//Updates the tab index showing a given tab as selected. 
//!DEPRECIATED, USE showProjectSelectedWithUID
showProjectSelectedForTabIndex:function(index) {
	//if(index != this.selectedProjectIndex) {
		this.selectedProjectIndex = index;
		this.view.visuallySelectTabWithIndex(index);
	//}
},

showProjectSelectedWithUID:function(uid) {
	this.selectedProjectIndex = this.view.visuallySelectTabWithTag(uid);
	
},

projectEditedAction:function() {
	this.showProjectGridView();
},

/*
	The current mode, could be
	1. Inbox
	2. Projects
	3. Smart
	4. Peeople...
	
	NOTE:	
	This is currently called after the task's have been redrawn with a call to -setTasks.
*/
setContextMode:function(str) {
	this.currentContext = str;
	
	if(str == 'Projects') {
		this.showsProjectTilesInRows = 0;
		this.view.showTabBar();
		
		if(this.projects.length < 1) {
			this.showProjectCreateView();
		}

	} else if(str == 'Inbox') {
		this.showsProjectTilesInRows = 0;
		this.view.hideTabBar();
		
		//Forces the list view.
		//TODO: This is dangurous and we need to sure that all edits are compelted if in an edit view.
		this.view.setTabViewPortView(this.taskListViewPort); 
	} else if(str == 'Search') {
		this.showsProjectTilesInRows = 1;
		this.view.hideTabBar();
	}
},

/*
	Updates the badge count for given project with id
*/
updateBadgeCountForProject:function(projectUid) {
	var project = TaskListController.projectWithUID_(projectUid);
	var tab = this.view.tabWithTag(projectUid);
	tab.setBadgeCount(project.valueForKey('dueCount'))
}

}});

// !ModelObject Proxy
fClass.NSObject.extend("ModelObject", { ivars: {
	uid:null,
	type:null
}, methods: {

initWithObject:function(obj, type) {
	if(this.performSuper(this, 'init')) {
		this.uid = obj.valueForKey('uid');
		//this.type = type;
	}
	return this;
},

valueForKey:function(key) {
	//if(key == 'uid') {
		return this.uid;
	/*} else {
		var obj = TaskListController.taskWithUID_(this.uid);
		if(obj.valueForKey) {
			return obj.valueForKey(key);
		} else {
			window.console.log('******** proxy correcting for access of key:' + key + ' obj:'+this.uid);
			return null;
		}
	}*/
}
	
}});

// !TaskInlinePromptView
/*
	Provides a prompt view that can be added to an task editor for various actions.
*/
fClass.WBView.extend("TaskInlinePromptView", { ivars : {

}, methods: {

initWithTitle:function(StringInstance) {
	if(this.initWithTag("div")) {
			
		this.addCSSClass("inline-prompt-view");
		this.setId('inline-prompt-view');
		
		this.titleView = fClass.WBView.alloc().initWithTag('span');
		this.titleView.addCSSClass('prompt-title');
		this.titleView.dom.html(StringInstance);
		this.addSubview(this.titleView);
		
	}
	return this;
}

}});


// !ConnectionPromptView
fClass.TaskInlinePromptView.extend("ConnectionPromptView", { ivars : {

task:null

}, methods :{

initWithTask:function(TaskInst) {
	var ownerName = TaskInst.valueForKey('ownerName');
	if(this.initWithTitle("Do you want to allow "+ownerName+" to share tasks with you?")) {
		this.task = TaskInst;
		
		this.setId("ConnectionPromptView");
		
		this.options = fClass.WBView.alloc().initWithTag('div');
		this.options.addCSSClass('prompt-options');
		
		this.options.okButton = fClass.WBControl.alloc().initWithTag('div');
		this.options.okButton.addCSSClass('button'); //Defined in WebViewContent.css
		this.options.okButton.dom.text('Yes');
		this.options.okButton.setFloatLeft();
		this.options.okButton.setTarget(this);
		this.options.okButton.setAction('acceptAction');
		this.options.addSubview(this.options.okButton);
		
		this.options.cancelButton = fClass.WBControl.alloc().initWithTag('div');
		this.options.cancelButton.addCSSClass('button'); //Defined in WebViewContent.css
		this.options.cancelButton.dom.text('No');
		this.options.cancelButton.setFloatLeft();
		this.options.cancelButton.setTarget(this);
		this.options.cancelButton.setAction('rejectAction');
		this.options.addSubview(this.options.cancelButton);
		
		this.options.reportAsSpamButton = fClass.WBControl.alloc().initWithTag('div');
		this.options.reportAsSpamButton.addCSSClass('link-button');
		this.options.reportAsSpamButton.dom.text('Report as Spam');
		this.options.reportAsSpamButton.setFloatRight();
		this.options.reportAsSpamButton.setTarget(this);
		this.options.reportAsSpamButton.setAction('reportAction');
		this.options.addSubview(this.options.reportAsSpamButton);
		
		this.addSubview(this.options);
	}
	return this;
},

acceptAction:function(Sender) {
	NSLog('should accept');
	TaskListController.acceptConnectionForTaskWithUID_(this.task.valueForKey('uid'));
	this.removeFromSuperview('fade', 300);
},

rejectAction:function(Sender) {
	TaskListController.rejectConnectionForTaskWithUID_(this.task.valueForKey('uid'));
},

reportAction:function(Sender) {
	TaskListController.reportAsSpamTaskWithUID_(this.task.valueForKey('uid'));
}

}});

// !AssignmentPromptView
fClass.TaskInlinePromptView.extend("AssignmentPromptView", { ivars : {

task:null

}, methods :{

initWithTask:function(TaskInst) {
	var ownerName = TaskInst.valueForKey('ownerName');
	if(this.initWithTitle("Task from "+ownerName+"")) {
		this.task = TaskInst;
		
		this.setId("AssignmentPromptView");
		
		this.options = fClass.WBView.alloc().initWithTag('div');
		this.options.setFloatRight();
		this.options.addCSSClass('prompt-options');
		
		this.options.okButton = fClass.WBControl.alloc().initWithTag('div');
		this.options.okButton.addCSSClass('button'); //Defined in WebViewContent.css
		this.options.okButton.dom.text('Accept');
		this.options.okButton.setFloatLeft();
		this.options.okButton.setTarget(this);
		this.options.okButton.setAction('acceptAction');
		this.options.addSubview(this.options.okButton);
		
		this.options.cancelButton = fClass.WBControl.alloc().initWithTag('div');
		this.options.cancelButton.addCSSClass('button'); //Defined in WebViewContent.css
		this.options.cancelButton.dom.text('Ignore');
		this.options.cancelButton.setFloatLeft();
		this.options.cancelButton.setTarget(this);
		this.options.cancelButton.setAction('rejectAction');
		this.options.addSubview(this.options.cancelButton);
		
		this.addSubview(this.options);
	}
	return this;
},

acceptAction:function(Sender) {
	//NSLog('should accept');
	TaskListController.acceptIncommingTaskWithUID_(this.task.valueForKey('uid'));
	this.removeFromSuperview('fade', 300);
},

rejectAction:function(Sender) {
	//TaskListController.rejectConnectionForTaskWithUID_(this.task.valueForKey('uid'));
	TaskListController.ignoreIncommingTaskWithUID_(this.task.valueForKey('uid'));
	//Gets deleted from view from cocoa side, closes the editor as well.
}

}});